<?php

/**
 * Description of SwpmCustomMessage
 */
class SwpmCustomMessage {

    public function __construct() {
        add_action('swpm_after_main_admin_menu', array(&$this, 'swpm_custom_msg_do_admin_menu'));
        add_action('admin_init', array(&$this, 'admin_init_hook'));
        add_filter('swpm_restricted_post_msg', array(&$this, 'swpm_restricted_post_msg'));
        add_filter('swpm_not_logged_in_post_msg', array(&$this, 'swpm_not_logged_in_post_msg'));
        add_filter('swpm_restricted_comment_msg', array(&$this, 'swpm_restricted_comment_msg'));
        add_filter('swpm_not_logged_in_comment_msg', array(&$this, 'swpm_not_logged_in_comment_msg'));
        add_filter('swpm_restricted_more_tag_msg', array(&$this, 'swpm_restricted_more_tag_msg'));
        add_filter('swpm_not_logged_in_more_tag_msg', array(&$this, 'swpm_not_logged_in_more_tag_msg'));

        add_filter('swpm_registration_success_msg', array(&$this, 'swpm_registration_success_msg'));
        add_filter('swpm_registration_email_activation_msg', array(&$this, 'swpm_registration_email_activation_msg'));

        add_filter('swpm_account_expired_msg', array(&$this, 'swpm_account_expired_msg'));

        //Full page protection addon
	add_filter('swpm_fpp_protected_content_msg',array(&$this,'swpm_fpp_protected_content_msg'));

        //Older posts protection addon
        add_filter('swpm_restricted_post_msg_older_post',array(&$this,'swpm_older_post_protected_content_msg'));
        add_filter('swpm_restricted_comment_older_post',array(&$this,'swpm_older_post_protected_content_msg'));

    }

    public function swpm_older_post_protected_content_msg($output) {
        $key = 'swpm_older_post_protected_msg';
        $msg = SwpmCustomMessageSettings::get_instance()->get_value($key);
        if( empty($msg)){
            //Nothing has been customized for this message. Use the default value.
            return $output;
        }

        //Perform shortcode processing on the custom message.
        $msg = do_shortcode($msg);

        global $post;
        if (isset($post->post_date)){
            //Replace the dynamic tag: {post_published_date}
            $formatted_translated_publish_date = SwpmUtils::get_formatted_and_translated_date_according_to_wp_settings($post->post_date);
            $msg = str_replace('{post_published_date}', $formatted_translated_publish_date, $msg);
        }
        return $msg;
    }

    public function swpm_fpp_protected_content_msg($output) {
        return $this->dispatch_message('swpm_fpp_protected_msg', $output);
    }

    public function swpm_registration_email_activation_msg($output){
        return $this->dispatch_message('swpm_registration_email_activation_msg', $output);
    }

    public function swpm_registration_success_msg($output) {
        return $this->dispatch_message('swpm_registration_success_msg', $output);
    }

    public function swpm_account_expired_msg($output) {
        return $this->dispatch_message('swpm_account_expired_msg', $output);
    }

    public function swpm_restricted_post_msg($output) {
        return $this->dispatch_message('swpm_restricted_post_msg', $output);
    }

    public function swpm_not_logged_in_post_msg($output) {
        return $this->dispatch_message('swpm_not_logged_in_post_msg', $output);
    }

    public function swpm_restricted_comment_msg($output) {
        return $this->dispatch_message('swpm_restricted_comment_msg', $output);
    }

    public function swpm_not_logged_in_comment_msg($output) {
        return $this->dispatch_message('swpm_not_logged_in_comment_msg', $output);
    }

    public function swpm_restricted_more_tag_msg($output) {
        return $this->dispatch_message('swpm_restricted_more_tag_msg', $output);
    }

    public function swpm_not_logged_in_more_tag_msg($output) {
        return $this->dispatch_message('swpm_not_logged_in_more_tag_msg', $output);
    }

    private function dispatch_message($key, $default) {
        $msg = SwpmCustomMessageSettings::get_instance()->get_value($key);
        return empty ($msg) ? $default : do_shortcode($msg);
    }

    public function swpm_custom_msg_do_admin_menu($menu_parent_slug) {
        add_submenu_page($menu_parent_slug, __("Custom Message", 'simple-membership'), __("Custom Message", 'simple-membership'), 'manage_options', 'swpm-custom-message', array(&$this, 'custom_message_admin_interface'));
    }

    public function custom_message_admin_interface() {
        $current_tab = SwpmCustomMessageSettings::get_instance()->current_tab;
        include(SWPM_CUSTOM_MSG_PATH . 'views/custom-message-settings.php');
    }

    public function admin_init_hook() {
        SwpmCustomMessageSettings::get_instance()->init_config_hooks();
    }

}
